package cn.com.qmth.stmms.ms.admin.service;

import cn.com.qmth.stmms.ms.core.domain.MarkStage;
import cn.com.qmth.stmms.ms.core.domain.MarkSubject;
import cn.com.qmth.stmms.ms.core.domain.Paper;
import cn.com.qmth.stmms.ms.core.domain.Work;
import cn.com.qmth.stmms.ms.core.domain.enums.TrialEnum;
import cn.com.qmth.stmms.ms.core.repository.*;
import cn.com.qmth.stmms.ms.core.vo.Subject;
import cn.com.qmth.stmms.ms.marking.service.StageControlService;
import com.alibaba.fastjson.JSONObject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * @Description: 试评任务
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2019/11/4
 */
@Service
public class TrialService {

    @Autowired
    WorkRepo workRepo;

    @Autowired
    MarkSubjectRepo markSubjectRepo;

    @Autowired
    StudentRepo studentRepo;

    @Autowired
    MarkTaskRepo markTaskRepo;

    @Autowired
    PaperRepo paperRepo;

    @Autowired
    StageControlService stageControlService;

    @Autowired
    ExamQuestionRepo examQuestionRepo;

    @Autowired
    MarkLogRepo markLogRepo;

    @Autowired
    MarkerGroupRepo markerGroupRepo;

    /**
     * 检查任务状态
     *
     * @param workId
     * @param subject
     * @throws Exception
     */
    @Transactional
    public void checkMissionStatus(Long workId, Subject subject) throws Exception {
        Work work = workRepo.findOne(workId);
        if (Objects.isNull(work)) {
            throw new Exception("没有此工作区，请检查workId是否正确");
        }
        List<MarkSubject> markSubjectList = work.getSubjects().stream().filter(o -> o.getId().toUpperCase().contains(subject.toString().toUpperCase()))
                .collect(Collectors.toList());
        MarkSubject markSubject = markSubjectList.get(0);
        if (markSubject.getTest() == TrialEnum.INIT.getId() || markSubject.getTest() == TrialEnum.START_TRIAL.getId()) {
            finishTrial(workId, subject);
        }
        //修改科目test
        markSubject.setTest(TrialEnum.START_FORMAL.getId());
        markSubject.setFormal(true);
        markSubjectRepo.save(markSubject);
    }

    /**
     * 开始试评任务
     *
     * @param workId
     * @param subject
     * @throws Exception
     */
    @Transactional
    public void startTrial(Long workId, Subject subject) throws Exception {
        Work work = workRepo.findOne(workId);
        if (Objects.isNull(work)) {
            throw new Exception("没有此工作区，请检查workId是否正确");
        }
        List<MarkSubject> markSubjectList = work.getSubjects().stream().filter(o -> o.getId().toUpperCase().contains(subject.toString().toUpperCase()))
                .collect(Collectors.toList());
        MarkSubject markSubject = markSubjectList.get(0);
        //分配任务
        stageControlService.goNext(markSubject, 0);
        //修改科目test
        markSubject.setTest(TrialEnum.START_TRIAL.getId());
        markSubjectRepo.save(markSubject);
        //先删除上次试评试卷(取标准卷用)
        paperRepo.deleteByWorkIdAndSubjectAndTest(workId, subject, TrialEnum.START_TRIAL.getId());
        //修改试卷test
        paperRepo.updateByWorkIdAndSubjectAndTest(workId, subject, TrialEnum.INIT.getId(), TrialEnum.START_TRIAL.getId());
    }

    /**
     * 删除试评任务，保留标准卷
     *
     * @param workId
     * @param subject
     * @throws Exception
     */
    @Transactional
    public void finishTrial(Long workId, Subject subject) throws Exception {
        Work work = workRepo.findOne(workId);
        if (Objects.isNull(work)) {
            throw new Exception("没有此工作区，请检查workId是否正确");
        }
        //1.更新科目为处始状态并设置test为false
        work.getSubjects().stream().filter(o -> o.getId().toUpperCase().contains(subject.toString().toUpperCase()))
                .collect(Collectors.toList())
                .forEach(o -> {
                    o.setTest(TrialEnum.DEFAULT.getId());
                    o.setStage(MarkStage.INIT);
                    o.setSampleCount(0);
                    markSubjectRepo.save(o);
                });
//        AtomicBoolean deleteStu = new AtomicBoolean(false);
//        work.getSubjects().forEach(o -> {
//            if (o.getId().toUpperCase().contains(subject.toString().toUpperCase())) {
//                o.setTest(TrialEnum.DEFAULT.getId());
//                o.setStage(MarkStage.INIT);
//                o.setTotalScore(0);
//                o.setSampleCount(0);
//            }
//            if (o.getTest() == TrialEnum.DEFAULT.getId()) {
//                deleteStu.set(true);
//            } else {
//                deleteStu.set(false);
//            }
//        });
//        markSubjectRepo.save(work.getSubjects());

        //2.删除该科目下所有任务数据
        markTaskRepo.deleteByWorkIdAndSubjectAndTest(workId, subject, TrialEnum.START_TRIAL.getId());
        //3.删除学生数据
        JSONObject jsonObject = new JSONObject();
        jsonObject.put(String.valueOf(TrialEnum.INIT.getId()), subject);
        studentRepo.deleteByWorkIdAndTest(workId, jsonObject.toJSONString());

        //4.删除卷子信息
        examQuestionRepo.deleteByWorkIdAndSubjectAndTest(workId, subject, TrialEnum.INIT.getId());
        //5.更新该科目下的试卷信息,可暂且不更新
        List<Paper> paperList = paperRepo.findByWorkIdAndSubjectAndTest(workId, subject, TrialEnum.START_TRIAL.getId());
        List<Long> paperIds = paperList.stream().map(o -> o.getId()).collect(Collectors.toList());
//        paperList.forEach(o -> {
//            o.setLevel(null);//更新档位
//            o.setScore(null);//更新分数
//        });
//        paperRepo.save(paperList);
        //5.删除学生的试卷信息(服务器图片)

        //6.删除log日志
        markLogRepo.deleteByPaperIdIn(paperIds);
        //7.删除分组
        markerGroupRepo.deleteBySubjectAndWorkId(subject, workId);
    }
}
